package binarytree.gui;

import java.util.ArrayList;
import java.util.HashMap;

import binarytree.core.BinarySearchTree;
import javafx.fxml.FXML;
import javafx.scene.layout.Pane;
import javafx.scene.shape.Line;
import javafx.scene.text.Text;
import javafx.scene.control.Button;
import javafx.scene.control.TextField;

public class BinaryTreeController {
	
	@FXML
	TextField insertValue;
	
	@FXML
	Pane visualTree;
	
	@FXML
	Button clear;
	
	BinarySearchTree<String> root = new BinarySearchTree<>();
	
	@FXML
	void initialize() {
		clear.setOnAction(event -> {for (String s: root.inorder()) {root.remove(s);} redrawTree();});
	}
	
	@FXML
	void insert() {
		if (insertValue.getText().length() > 0) {
			root.insert(insertValue.getText());
			redrawTree();
			insertValue.setText("");
		}
	}
	
	void redrawTree() {
		HashMap<String,Text> string2text = new HashMap<>();
		visualTree.getChildren().clear();
		ArrayList<ArrayList<ArrayList<String>>> levels = root.levelOrder();
		double xOffset = visualTree.getWidth() / (1 + maxWidth(levels));
		double xCenter = (xOffset + visualTree.getWidth()) / 2;
		double yOffset = visualTree.getHeight() / levels.size();
		for (int level = levels.size() - 1; level >= 0; level -= 1) {
			int midpoint = (levels.get(level).size()+1) / 2;
			for (int column = 0; column < levels.get(level).size(); column++) {
				ArrayList<String> nodeAndKids = levels.get(level).get(column);
				if (nodeAndKids.size() > 0) {
					Text nodeText = new Text(nodeAndKids.get(0));
					string2text.put(nodeText.getText(), nodeText);
					nodeText.setX(xCenter + xOffset * (column - midpoint));
					nodeText.setY(yOffset * (0.5 + level));
					visualTree.getChildren().add(nodeText);
					for (int kid = 1; kid < nodeAndKids.size(); kid++) {
						connect(nodeText, string2text.get(nodeAndKids.get(kid)));
					}
					nodeText.setOnMouseClicked(event -> {
						root.remove(nodeText.getText());
						redrawTree();
					});
				}
			}
		}
	}
	
	static <T> int maxWidth(ArrayList<ArrayList<T>> levels) {
		int max = 0;
		for (ArrayList<T> level: levels) {
			max = Math.max(max, level.size());
		}
		return max;
	}
	
	void connect(Text one, Text two) {
		Line connection = new Line(one.getX(), one.getY(), two.getX(), two.getY());
		visualTree.getChildren().add(connection);
	}
}
